{
 How to drag text from TStringGrid to TSRichViewEdit

 This demo uses VCL drag & drop:
 - BeginDrag for the source component (grid)
 - OnDragOver and OnDragDrop for the target component (editor)
}

unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, Dialogs, Grids,
  RVScroll, RichView, RVEdit, CRVFData, SclRView;

const
  WM_SETCARETPOS = WM_USER + 10;

type
  TfrmMain = class(TForm)
    StringGrid1: TStringGrid;
    SRichViewEdit1: TSRichViewEdit;
    procedure FormCreate(Sender: TObject);
    procedure SRichViewEdit1DragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure SRichViewEdit1DragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure StringGrid1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure StringGrid1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    { Private declarations }
    ClickPoint: TPoint;
    procedure WMSetCaretPos(var Msg: TMessage); message WM_SETCARETPOS;
  public
    { Public declarations }
    procedure MoveCaretTo(X, Y: Integer);
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  StringGrid1.Cells[0,0] := 'AA';
  StringGrid1.Cells[1,0] := 'AB';
  StringGrid1.Cells[2,0] := 'AC';
  StringGrid1.Cells[0,1] := 'BA';
  StringGrid1.Cells[1,1] := 'BB';
  StringGrid1.Cells[2,1] := 'BC';
end;


procedure TfrmMain.StringGrid1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  Col, Row: Integer;
begin
  // When left-clicked to a cell, remembering click coordinates
  if Button = mbLeft then
  begin
    StringGrid1.MouseToCell(X, Y, Col, Row);
    if (Col >= 0) and (Row >= 0) then
      ClickPoint := Point(X, Y)
    else
      ClickPoint := Point(-1, -1)
  end;
end;

procedure TfrmMain.StringGrid1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  // Starting drag if mouse pointer was moved from the click point
  if (ssLeft in Shift) and (ClickPoint.X >= 0) and
    (Sqr(ClickPoint.X - X) + Sqr(ClickPoint.Y - Y) > 5) then
  begin
    StringGrid1.BeginDrag(True);
  end;
end;

procedure TfrmMain.SRichViewEdit1DragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := Source is TStringGrid;
  // We cannot create/destroy inplace editors for cells just
  // in OnDragOver, so we position caret in OnWMSetCaretPos
  if Accept then
    PostMessage(Handle, WM_SETCARETPOS, X, Y);
end;

procedure TfrmMain.SRichViewEdit1DragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
  // Accepting text from the grid
  if not (Source is TStringGrid) then
    exit;
  MoveCaretTo(X, Y);
  SRichViewEdit1.ActiveEditor.InsertText(
    StringGrid1.Cells[StringGrid1.Col, StringGrid1.Row]);
end;

procedure TfrmMain.WMSetCaretPos(var Msg: TMessage);
begin
  MoveCaretTo(Msg.WParam, Msg.LParam);
end;


procedure TfrmMain.MoveCaretTo(X, Y: Integer);
var
  RVData:                       TCustomRVFormattedData;
  ItemNo, Offs:                 Integer;
  p:                            TPoint;
  InArea, bInLeft, bInPageArea: Boolean;
begin
  p := SRichViewEdit1.ConvertSRVtoRV(Point(X, Y), InArea, bInLeft, bInPageArea);
  p := SRichViewEdit1.ActiveEditor.ClientToDocument(p);
  SRichViewEdit1.ActiveEditor.GetItemAt(p.X, p.Y, RVData, ItemNo, Offs, False);
  RVData := RVData.Edit as TCustomRVFormattedData;
  if ItemNo < 0 then
    exit;
  RVData.SetSelectionBounds(ItemNo, Offs, ItemNo, Offs);
  RVData.Invalidate;
end;




end.

